@@ -556,7 +556,7 @@ get_exec_overridden_desktop_file (const char *exec_string)
   return result;
 }
 
-char *
+static char *
 get_env_overridden_desktop_file (guint pid)
 {
   gchar *environ_file;
@@ -827,7 +827,6 @@ load_desktop_file_to_table (BamfMatcher * self,
 {
   GDesktopAppInfo *desktop_file;
   gboolean no_display;
-  const char *current_desktop;
   char *exec;
   char *path;
   GString *desktop_id; /* is ok... really */
@@ -841,9 +840,7 @@ load_desktop_file_to_table (BamfMatcher * self,
       return;
     }
 
-  current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
-
-  if (current_desktop && !g_desktop_app_info_get_show_in (desktop_file, current_desktop))
+  if (!g_desktop_app_info_get_show_in (desktop_file, NULL))
     {
       g_object_unref (desktop_file);
       return;
@@ -945,7 +942,8 @@ load_index_file_to_table (BamfMatcher * self,
   GDataInputStream *input;
   char *line;
   char *directory;
-  const char *current_desktop;
+  gchar **current_desktops = NULL;
+  const gchar *xdg_current_desktop;
   gsize length;
 
   file = g_file_new_for_path (index_file);
@@ -961,13 +959,15 @@ load_index_file_to_table (BamfMatcher * self,
     }
 
   length = 0;
-  current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
+
+  xdg_current_desktop = g_getenv ("XDG_CURRENT_DESKTOP");
+
+  if (xdg_current_desktop)
+    current_desktops = g_strsplit (xdg_current_desktop, ":", 0);
+
   directory = g_path_get_dirname (index_file);
   input = g_data_input_stream_new (G_INPUT_STREAM (stream));
 
-  if (current_desktop && current_desktop[0] == '\0')
-    current_desktop = NULL;
-
   while ((line = g_data_input_stream_read_line (input, &length, NULL, NULL)))
     {
       char *exec;
@@ -982,7 +982,7 @@ load_index_file_to_table (BamfMatcher * self,
 
       show_in = parts[3];
 
-      if (current_desktop && show_in && show_in[0] != '\0')
+      if (current_desktops && show_in && show_in[0] != '\0')
         {
           gchar **sub_parts = g_strsplit (show_in, ";", -1);
           gboolean found_current = FALSE;
@@ -990,7 +990,7 @@ load_index_file_to_table (BamfMatcher * self,
 
           for (i = 0; sub_parts[i]; ++i)
             {
-              if (g_ascii_strcasecmp (sub_parts[i], current_desktop) == 0)
+              if (g_strv_contains ((const gchar * const *) current_desktops, sub_parts[i]) == 0)
                 {
                   found_current = TRUE;
                   break;
@@ -1041,6 +1041,7 @@ load_index_file_to_table (BamfMatcher * self,
   g_object_unref (input);
   g_object_unref (stream);
   g_object_unref (file);
+  g_strfreev (current_desktops);
   g_free (directory);
 }
 
@@ -1734,7 +1735,7 @@ bamf_matcher_possible_applications_for_window (BamfMatcher *self,
 
       if (!desktop_files)
         {
-          app_id = bamf_legacy_window_get_hint (window, _GTK_APPLICATION_ID);
+          app_id = bamf_window_get_application_id (bamf_window);
 
           if (app_id)
             {
