@@ -28,6 +28,8 @@ G_DEFINE_TYPE_WITH_CODE (BamfView, bamf_view, BAMF_DBUS_ITEM_TYPE_OBJECT_SKELETO
                          G_IMPLEMENT_INTERFACE (BAMF_DBUS_ITEM_TYPE_VIEW,
                                                 bamf_view_dbus_view_iface_init));
 
+#define STARTING_MAX_WAIT 15
+
 enum
 {
   PROP_0,
@@ -35,6 +37,7 @@ enum
   PROP_NAME,
   PROP_ICON,
   PROP_ACTIVE,
+  PROP_STARTING,
   PROP_RUNNING,
   PROP_URGENT,
   PROP_USER_VISIBLE,
@@ -43,8 +46,6 @@ enum
 enum
 {
   CLOSED_INTERNAL,
-  CHILD_ADDED_INTERNAL,
-  CHILD_REMOVED_INTERNAL,
   EXPORTED,
 
   LAST_SIGNAL,
@@ -57,6 +58,7 @@ typedef struct _BamfViewPropCache
   /* FIXME: temporary cache these properties until we don't export the view
    * to the bus, we need this until the skeleton won't be smart enough to emit
    * signals as soon as the object is exported */
+  gboolean starting;
   gboolean running;
   gboolean user_visible;
   gboolean urgent;
@@ -74,6 +76,9 @@ struct _BamfViewPrivate
   GList * children;
   GList * parents;
   gboolean closed;
+  guint starting_timeout;
+
+  SnStartupSequence *startup_sequence;
 
   /* FIXME: remove this as soon as we move to properties on library as well */
   guint active_changed_idle;
@@ -112,6 +117,9 @@ bamf_view_active_changed (BamfView *view, gboolean active)
       guint idle = g_idle_add_full (G_PRIORITY_DEFAULT, on_active_changed_idle, view, NULL);
       view->priv->active_changed_idle = idle;
     }
+
+  if (active)
+    bamf_view_set_starting (view, NULL, FALSE);
 }
 
 static void
@@ -144,6 +152,41 @@ bamf_view_user_visible_changed (BamfView *view, gboolean user_visible)
     g_signal_emit_by_name (view, "user-visible-changed", user_visible);
 }
 
+static gboolean
+on_starting_timeout (gpointer data)
+{
+  BamfView *view = data;
+
+  bamf_view_set_starting (view, NULL, FALSE);
+  view->priv->starting_timeout = 0;
+
+  return FALSE;
+}
+
+static void
+bamf_view_starting_changed (BamfView *view, gboolean starting)
+{
+  BamfViewPrivate *priv;
+
+  g_return_if_fail (BAMF_IS_VIEW (view));
+
+  priv = view->priv;
+
+  if (BAMF_VIEW_GET_CLASS (view)->starting_changed)
+    {
+      BAMF_VIEW_GET_CLASS (view)->starting_changed (view, starting);
+    }
+
+  if (priv->starting_timeout)
+    {
+      g_source_remove (priv->starting_timeout);
+      priv->starting_timeout = 0;
+    }
+
+  if (starting)
+    priv->starting_timeout = g_timeout_add_seconds (STARTING_MAX_WAIT, on_starting_timeout, view);
+}
+
 static void
 bamf_view_running_changed (BamfView *view, gboolean running)
 {
@@ -157,6 +200,9 @@ bamf_view_running_changed (BamfView *view, gboolean running)
 
   if (emit)
     g_signal_emit_by_name (view, "running-changed", running);
+
+  if (running)
+    bamf_view_set_starting (view, NULL, FALSE);
 }
 
 static void
@@ -172,6 +218,9 @@ bamf_view_urgent_changed (BamfView *view, gboolean urgent)
 
   if (emit)
     g_signal_emit_by_name (view, "urgent-changed", urgent);
+
+  if (urgent)
+    bamf_view_set_starting (view, NULL, FALSE);
 }
 
 void
@@ -299,6 +348,13 @@ bamf_view_handle_child_closed (BamfView *child,
   bamf_view_remove_child (view, child);
 }
 
+static void
+on_child_view_exported (BamfView *child, BamfView *view)
+{
+  g_signal_emit_by_name (view, "child-added", bamf_view_get_path (child));
+  g_signal_handlers_disconnect_by_func (child, on_child_view_exported, view);
+}
+
 void
 bamf_view_add_child (BamfView *view,
                      BamfView *child)
@@ -309,20 +365,26 @@ bamf_view_add_child (BamfView *view,
   g_return_if_fail (BAMF_IS_VIEW (child));
 
   g_signal_connect (G_OBJECT (child), "closed-internal",
-                    (GCallback) bamf_view_handle_child_closed, view);
+                    G_CALLBACK (bamf_view_handle_child_closed), view);
 
   /* Make sure our parent child lists are ok, pay attention to whose list you add parents to */
   view->priv->children = g_list_prepend (view->priv->children, child);
   child->priv->parents = g_list_prepend (child->priv->parents, view);
 
+  if (bamf_view_is_on_bus (child))
+    {
+      added = bamf_view_get_path (child);
+      g_signal_emit_by_name (view, "child-added", added);
+    }
+  else
+    {
+      g_signal_connect (G_OBJECT (child), "exported",
+                        G_CALLBACK (on_child_view_exported), view);
+    }
+
   // Do this by hand so we can pass and object instead of a string
   if (BAMF_VIEW_GET_CLASS (view)->child_added)
     BAMF_VIEW_GET_CLASS (view)->child_added (view, child);
-
-  g_signal_emit (view, view_signals[CHILD_ADDED_INTERNAL], 0, child);
-
-  added = bamf_view_get_path (child);
-  g_signal_emit_by_name (view, "child-added", added);
 }
 
 void
@@ -333,14 +395,12 @@ bamf_view_remove_child (BamfView *view, BamfView *child)
   g_return_if_fail (BAMF_IS_VIEW (view));
   g_return_if_fail (BAMF_IS_VIEW (child));
 
-  g_signal_handlers_disconnect_by_func (child, bamf_view_handle_child_closed, view);
+  g_signal_handlers_disconnect_by_data (child, view);
 
   /* Make sure our parent child lists are ok, pay attention to whose list you add parents to */
   view->priv->children = g_list_remove (view->priv->children, child);
   child->priv->parents = g_list_remove (child->priv->parents, view);
 
-  g_signal_emit (view, view_signals[CHILD_REMOVED_INTERNAL], 0, child);
-
   removed = bamf_view_get_path (child);
   g_signal_emit_by_name (view, "child-removed", removed);
 
@@ -420,6 +480,44 @@ bamf_view_set_urgent (BamfView *view,
   BAMF_VIEW_SET_BOOL_PROPERTY (view, urgent);
 }
 
+gboolean
+bamf_view_is_starting (BamfView *view)
+{
+ BAMF_VIEW_GET_PROPERTY (view, starting, FALSE);
+}
+
+void
+bamf_view_set_starting (BamfView *view,
+                        SnStartupSequence *startup_sequence,
+                        gboolean starting)
+{
+  if (!bamf_view_is_starting (view) && starting)
+    {
+      if (view->priv->startup_sequence)
+      {
+        sn_startup_sequence_unref (view->priv->startup_sequence);
+        view->priv->startup_sequence = NULL;
+      }
+
+      if (startup_sequence)
+        {
+          view->priv->startup_sequence = startup_sequence;
+          sn_startup_sequence_ref (view->priv->startup_sequence);
+        }
+    }
+  else if (!starting)
+    {
+      if (view->priv->startup_sequence)
+        {
+          sn_startup_sequence_complete (view->priv->startup_sequence);
+          sn_startup_sequence_unref (view->priv->startup_sequence);
+          view->priv->startup_sequence = NULL;
+        }
+    }
+
+  BAMF_VIEW_SET_BOOL_PROPERTY (view, starting);
+}
+
 gboolean
 bamf_view_is_running (BamfView *view)
 {
@@ -516,6 +614,7 @@ bamf_view_cached_properties_notify (BamfView *view)
   bamf_view_set_name (view, cache->name);
   bamf_view_set_icon (view, cache->icon);
   bamf_view_set_active (view, cache->active);
+  bamf_view_set_starting (view, NULL, cache->starting);
   bamf_view_set_running (view, cache->running);
   bamf_view_set_user_visible (view, cache->user_visible);
   bamf_view_set_urgent (view, cache->urgent);
@@ -785,12 +884,24 @@ bamf_view_dispose (GObject *object)
       priv->parents = NULL;
     }
 
+  if (priv->starting_timeout)
+    {
+      g_source_remove (priv->starting_timeout);
+      priv->starting_timeout = 0;
+    }
+
   if (priv->active_changed_idle)
     {
       g_source_remove (priv->active_changed_idle);
       priv->active_changed_idle = 0;
     }
 
+  if (priv->startup_sequence)
+    {
+      sn_startup_sequence_unref (priv->startup_sequence);
+      priv->startup_sequence = NULL;
+    }
+
   bamf_view_cached_properties_clear (view);
   g_dbus_object_skeleton_flush (G_DBUS_OBJECT_SKELETON (view));
 
@@ -829,6 +940,9 @@ bamf_view_get_property (GObject *object, guint property_id, GValue *value, GPara
       case PROP_USER_VISIBLE:
         g_value_set_boolean (value, bamf_view_is_user_visible (view));
         break;
+      case PROP_STARTING:
+        g_value_set_boolean (value, bamf_view_is_starting (view));
+        break;
       case PROP_RUNNING:
         g_value_set_boolean (value, bamf_view_is_running (view));
         break;
@@ -856,6 +970,8 @@ bamf_view_init (BamfView * self)
   self->priv->dbus_iface = _bamf_dbus_item_view_skeleton_new ();
   self->priv->props = g_new0 (BamfViewPropCache, 1);
 
+  self->priv->startup_sequence = NULL;
+
   /* We need to connect to the object own signals to redirect them to the dbus
    * interface                                                                */
   g_signal_connect (self, "active-changed", G_CALLBACK (on_view_active_changed), NULL);
@@ -923,6 +1039,7 @@ bamf_view_class_init (BamfViewClass * klass)
   g_object_class_override_property (object_class, PROP_ICON, "icon");
   g_object_class_override_property (object_class, PROP_ACTIVE, "active");
   g_object_class_override_property (object_class, PROP_URGENT, "urgent");
+  g_object_class_override_property (object_class, PROP_STARTING, "starting");
   g_object_class_override_property (object_class, PROP_RUNNING, "running");
   g_object_class_override_property (object_class, PROP_USER_VISIBLE, "user-visible");
 
@@ -934,22 +1051,6 @@ bamf_view_class_init (BamfViewClass * klass)
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
-  view_signals [CHILD_ADDED_INTERNAL] =
-    g_signal_new ("child-added-internal",
-                  G_OBJECT_CLASS_TYPE (klass),
-                  G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (BamfViewClass, child_added_internal),
-                  NULL, NULL, NULL,
-                  G_TYPE_NONE, 1, BAMF_TYPE_VIEW);
-
-  view_signals [CHILD_REMOVED_INTERNAL] =
-    g_signal_new ("child-removed-internal",
-                  G_OBJECT_CLASS_TYPE (klass),
-                  G_SIGNAL_RUN_FIRST,
-                  G_STRUCT_OFFSET (BamfViewClass, child_removed_internal),
-                  NULL, NULL, NULL,
-                  G_TYPE_NONE, 1, BAMF_TYPE_VIEW);
-
   view_signals [EXPORTED] =
     g_signal_new ("exported",
                   G_OBJECT_CLASS_TYPE (klass),
